﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.ServiceModel.Security;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using Syncfusion.UI.Xaml.Diagram;
using System.Collections.ObjectModel;
// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238
using Syncfusion.UI.Xaml.Diagram.Controls;

namespace NestedObjects_115429
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        public MainPage()
        {
            this.InitializeComponent();
            (Diagram.SelectedItems as SelectorViewModel).QuickCommands = Visibility.Collapsed;
            ObservableCollection<CustomNode> nodes = new ObservableCollection<CustomNode>();
            ObservableCollection<CustomGroup> groups = new ObservableCollection<CustomGroup>();
            CustomNode n1 = CreateNodes(100, 100);
            CustomNode n2 = CreateNodes(300, 300);
            CustomNode n3 = CreateNodes(400, 400);
            CustomNode n4 = CreateNodes(500, 500);
            (Diagram.Info as IGraphInfo).ItemDoubleTappedEvent += MainPage_ItemDoubleTappedEvent;
            (Diagram.Info as IGraphInfo).NodeChangedEvent += MainPage_NodeChangedEvent;
            CustomGroup g = new CustomGroup()
            {
                Nodes = new ObservableCollection<object>()
                {
                   n1,n2
                }
            };
            nodes.Add(n1);
            nodes.Add(n2);

            nodes.Add(n3);
            nodes.Add(n4);
          
            Diagram.Nodes = nodes;
            groups.Add(g);
            Diagram.Groups = groups;
        }

        void MainPage_NodeChangedEvent(object sender, ChangeEventArgs<object, NodeChangedEventArgs> args)
        {
            if (args.Item is CustomNode)
            {
                CustomNode c = args.Item as CustomNode;
                if (c.ParentGroup != null && c.IsCollapsed)
                {
                    c.X += args.NewValue.OffsetX - args.OldValue.OffsetX;
                    c.Y += args.NewValue.OffsetY - args.OldValue.OffsetY;
                }
                
            }
        }

        void MainPage_ItemDoubleTappedEvent(object sender, DiagramEventArgs args)
        {
            CustomNode node;
            if (args.Item is CustomNode)
            {
                node = args.Item as CustomNode;
            }
            else
            {
                node = (args.Item as CustomGroup).Nodes.Any() ? (args.Item as CustomGroup).Nodes.FirstOrDefault() as CustomNode: null;
            }

            if (node!=null && !node.IsCollapsed && node.ParentGroup!=null)
            {
                CustomGroup g = node.ParentGroup as CustomGroup;
                double x = 0, y = 0;
                x = (g.Info as IGroupInfo).ActualWidth / 2 - 100;
                y = (g.Info as IGroupInfo).ActualHeight / 2 - 100;
                (Diagram.Info as IGraphInfo).Commands.BringToFront.Execute(g);
                g.BackGround = new SolidColorBrush(Colors.Purple);
                foreach (var item in (Diagram.Groups.First() as CustomGroup).Nodes)
                {
                    CustomNode c = (item as CustomNode);
                    c.X = c.OffsetX;
                    c.OffsetX = g.OffsetX - x;
                    c.Y = c.OffsetY;
                    c.OffsetY = g.OffsetY - y;
                    c.H = c.UnitHeight;
                    c.W = c.UnitWidth;
                    c.UnitWidth = 200;
                    c.UnitHeight = 200;
                    c.IsCollapsed = true;
                }
            }
            else if(node!=null && node.ParentGroup!=null)
            {
                CustomGroup g = node.ParentGroup as CustomGroup;
                g.BackGround = new SolidColorBrush(Colors.Transparent);
                foreach (var item in (Diagram.Groups.First() as CustomGroup).Nodes)
                {
                    CustomNode c = (item as CustomNode);
                    c.OffsetX = c.X;
                    c.OffsetY = c.Y;
                    c.UnitWidth = c.W;
                    c.UnitHeight = c.H;
                    c.IsCollapsed = false;
                }
            }
        }

        private CustomNode CreateNodes(double x, double y)
        {
            return new CustomNode()
            {
                UnitWidth = 100,
                UnitHeight = 100,
                OffsetX = x,
                OffsetY = y,
                //Unique Guid for NodeViewModel
                ID = Guid.NewGuid(),

                //Shape for NodeViewModel
                Shape = new RectangleGeometry() { Rect = new Rect(0, 0, 10, 10) },

                //ShapeStyle
                ShapeStyle = this.Resources["shapestyle"] as Style,
            }; 
        }

        /// <summary>
        /// Invoked when this page is about to be displayed in a Frame.
        /// </summary>
        /// <param name="e">Event data that describes how this page was reached.  The Parameter
        /// property is typically used to configure the page.</param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
        }
    }

    public class CustomNode : NodeViewModel
    {
        public bool IsCollapsed = false;
        private double _x;

        public double X
        {
            get { return _x; }
            set
            {
                _x = value;
                OnPropertyChanged("X");
            }
        }

        private double _y;

        public double Y
        {
            get { return _y; }
            set
            {
                _y = value;
                OnPropertyChanged("Y");
            }
        }

        private double _w;

        public double W
        {
            get { return _w; }
            set
            {
                _w = value;
                OnPropertyChanged("W");
            }
        }

        private double _h;

        public double H
        {
            get { return _h; }
            set
            {
                _h = value;
                OnPropertyChanged("H");
            }
        }
    }

    public class CustomGroup : GroupViewModel
    {
        private SolidColorBrush _backgroundBrush;

        public SolidColorBrush BackGround
        {
            get { return _backgroundBrush; }
            set
            {
                _backgroundBrush = value;
                OnPropertyChanged("BackGround");
            }
        }
    }

    public class CustomDiagram : SfDiagram
    {
        protected override Node GetNodeForItemOverride(object item)
        {
            Node g = new Node();
            Binding visi = new Binding();
            visi.Source = (item as CustomNode);
            visi.Path = new PropertyPath("Visibility");
            g.SetBinding(VisibilityProperty, visi);
            return g;
        }

        protected override Group GetGroupForItemOverride(object item)
        {
            Group g=new Group();
            Binding background=new Binding();
            background.Source = (item as CustomGroup);
            background.Path=new PropertyPath("BackGround");
            g.SetBinding(BackgroundProperty,background);
            //Canvas.SetZIndex(g,-1);
            return g;
        }
    }
}
